import getpass, sys

from envbash import load_envbash
from izzylib import DotDict, Path, boolean, logging
from os import getcwd, environ as env


scriptpath = Path(__file__).resolve.parent

path = DotDict(
	script = scriptpath,
	data = env.get('SYNC_DATA', Path(getcwd()).join('data')),
	frontend = scriptpath.join('frontend')
)

path.data.mkdir()

try:
	load_envbash(path.data.join('production.env'))
except FileNotFoundError:
	logging.warning('No environment file found. Creating a new one and exiting')

	with path.data.join('production.env').open('w') as fd:
		fd.write(f"""# Server settings
SYNC_LISTEN=localhost
SYNC_PORT=8080
SYNC_HOST=localhost:8080
#SYNC_ALT_HOSTS=example.com,sync.example.com
SYNC_PROTO=http
SYNC_LOG_LEVEL=info

# Database settings
SYNC_DB_TYPE=sqlite
SYNC_DB_HOST=/var/run/postgresql
SYNC_DB_PORT=5432
SYNC_DB_NAME=pywebsync
SYNC_DB_USER={getpass.getuser()}
#SYNC_DB_PASS=changeme
SYNC_DB_SSL=False
""")

	sys.exit()


config = DotDict(
	listen = env.get('SYNC_LISTEN', 'localhost'),
	port = int(env.get('SYNC_PORT', 8080)),
	proto = env.get('SYNC_PROTO', 'http'),
	workers = int(env.get('SYNC_WORKERS', 1)),
	alt_hosts = env.get('SYNC_ALT_HOSTS', '').split(',')
)


if env.get('SYNC_DB_TYPE', 'sqlite').lower() == 'sqlite':
	dbconfig = DotDict(
		name = path.data.join('database.db')
	)

else:
	dbconfig = DotDict({
		'host': env.get('SYNC_DB_HOST', '/var/run/postgresql'),
		'port': int(env.get('SYNC_DB_PORT', 5432)),
		'name': env.get('SYNC_DB_NAME', 'pywebsync'),
		'user': env.get('SYNC_DB_USER', getpass.getuser()),
		'pass': env.get('SYNC_DB_PASS'),
		'ssl': boolean(env.get('SYNC_DB_SSL'))
	})

config.host = env.get('SYNC_HOST', f'{config.listen}:{config.port}')
logging.set_config('level', env.get('SYNC_LOG_LEVEL', 'info'))
