import sys

from functools import partial
from izzylib import DotDict, boolean, logging
from izzylib.sql import OperationalError, SqlDatabase, SqlSession, SqlColumn as Col

from .get_func import Get
from .put_func import Put
from .rem_func import Rem

from ..config import dbconfig


version = 20210821
tables = DotDict(
	config = [
		Col('id'),
		Col('key', 'text', nullable=False),
		Col('value', 'text', nullable=False)
	],
	user = [
		Col('id'),
		Col('handle', 'text', nullable=False),
		Col('display', 'text'),
		Col('password', 'text', nullable=False),
		Col('level', 'integer', default=10),
		Col('config', 'json', default={})
	],
	browser = [
		Col('id'),
		Col('userid', 'integer', nullable=False, fkey='user.id'),
		Col('uuid', 'text', nullable=False),
		Col('useragent', 'text'),
		Col('address', 'text'),
		Col('pubkey', nullable=False),
		Col('last_seen', 'datetime'),
		Col('timestamp')
	],
	action = [
		Col('id'),
		Col('action', 'text', nullable=False),
		Col('data', 'json', nullable=False),
		Col('userid', 'integer', nullable=False, fkey='user.id'),
		Col('browserid', 'integer', nullable=False, fkey='browser.id'),
		Col('timestamp')
	],
	token = [
		Col('id'),
		Col('code', 'text', nullable=False),
		Col('userid', 'integer', nullable=False, fkey='user.id'),
		Col('last_seen', 'datetime'),
		Col('timestamp')
	],
	fedi_acct = [
		Col('id'),
		Col('userid', 'text', nullable=False, fkey='user.id'),
		Col('handle', 'text', nullable=False),
		Col('domain', 'text', nullable=False),
		Col('token', 'text', nullable=False),
		Col('last_updated', 'datetime'),
		Col('timestamp')
	]
)


class Database(SqlDatabase):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs, session_class=Session)

		self.get = partial(self.session_func, 'get')
		self.put = partial(self.session_func, 'put')
		self.rem = partial(self.session_func, 'rem')


	def session_func(self, action, func, *args, **kwargs):
		with db.session as s:
			return s[action][func](*args, **kwargs)


class Session(SqlSession):
	config_defaults = dict(
		version = (0, int),
		enable_registration = (True, boolean)
	)

	# Don't need this yet
	config_store_func = dict()


	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		self.get = Get(self)
		self.put = Put(self)
		self.rem = Rem(self)


	def __getitem__(self, key):
		return getattr(self, key)


db = Database(tables=tables, **dbconfig)
