from datetime import datetime, timedelta
from izzylib.hasher import PasswordHasher


hasher = PasswordHasher(iterations=64)


class SessionFunctionBase:
	def __init__(self, session):
		self.session = session
		self.database = session.database
		self.cache = self.database.cache
		self.defaults = session.config_defaults

		# aliases
		self.s = session
		self.db = session.database


	def __getitem__(self, key):
		return getattr(self, key)


class ItemBase:
	def __init__(self, db, row):
		self.db = db
		self.row = row

		for k,v in row.items():
			setattr(self, k, v)


	def __getitem__(self, key):
		return self.row[key]


	def __setitem__(self, key, value):
		self.row[key]
		self.row[key] = value


class Token(ItemBase):
	_user = None

	def get_cookie_data(self, **kwargs):
		return DotDict(
			expires = kwargs.get('expires', self.last_access+timedelta(weeks=2)),
			samesite = kwargs.get('samesite', 'strict'),
			https = kwargs.get('https', True),
			secure = kwargs.get('secure', True)
		)


	def set_cookie_data(self, response, **kwargs):
		data = self.get_cookie_data()

		if response.__class__.__name__ == 'HTTPResponse':
			response.cookies['token'] = self.code
			response.cookies['token']['expires'] = data.expires
			response.cookies['token']['samesite'] = data.samesite
			response.cookies['token']['https'] = data.https
			response.cookies['token']['secure'] = data.secure

		else:
			response.set_cookie('token', self.code, **data)

	def save_access_time(self):
		with self.db.session as s:
			self.save_access_time_session(s)


	def save_access_time_session(self, s):
		s.update(row=self.row, last_access=self.last_access)


	def set_cookie(self, response):
		self.set_cookie_data(response)


	def del_cookie(self, response):
		self.set_cookie_data(response, expires=datetime.fromtimestamp(0))
		self.db.rem('token', self['code'])


	@property
	def user(self):
		if not self._user:
			with self.db.session as s:
				self._user = s.fetch('user', id=self.userid)

		return self._user


class User(ItemBase):
	@property
	def accounts(self):
		return self.get_data('fedi_acct')


	@property
	def actions(self):
		return self.get_data('action')


	@property
	def browsers(self):
		return self.get_data('browser')


	@property
	def tokens(self):
		return self.get_data('token')


	def get_data(self, table):
		with self.db.session as s:
			return s.fetch(table, userid=self.row.id)
