from datetime import datetime, timedelta
from izzylib import random_gen

from .func_base import SessionFunctionBase, Token, User, hasher


class Put(SessionFunctionBase):
	def config(self, key, value):
		row = self.s.fetch('config', key=key)

		if row:
			row = self.s.update(row=row, value=value, return_row=True)

		else:
			row = self.s.insert('config', key=key, value=value, return_row=True)

		self.cache.config.store(key, value)
		return value


	def token(self, userid, token=None):
		if not token:
			token = random_gen(40)
			row = None

		else:
			row = self.s.get.token(token)

		now = datetime.now()

		if row:
			row = self.s.update(row=row, last_seen=now, return_row=True)

		else:
			row = self.s.insert('token', userid=userid,
				code = random_gen(40),
				last_seen = now,
				timestamp = now,
				return_row=True
			)

		token = Token(self.db, row)
		self.cache.token.store(token, token)
		return token


	# Completely forgot to hash the password. This'll be fixed later
	def user(self, handle, password, display=None):
		user = self.s.get.user(handle)
		display = display or handle
		passhash = hasher.hash(password)

		if user:
			row = self.s.update(row=user.row, password=passhash, display=display, return_row=True)

		else:
			row = self.s.insert('user',
				handle = handle,
				password = passhash,
				display = display,
				config = {},
				return_row=True
			)

		user = User(self.db, row)
		self.cache.user.store(handle, user)
		return user
