import random

from izzylib import DotDict, Path, http_server
from sicaptcha.sicaptcha import Sicaptcha
from urllib.request import urlopen

from .config import path


font_path = Path(http_server.__file__).resolve.parent.join('frontend/static/nunito/NunitoSans-ExtraBold.ttf')


class Captcha(Sicaptcha):
	word_cache_path = path.data.join('word_cache')
	word_cache_path_local = Path('/etc/dictionaries-common/words')
	word_cache = []

	defaults = {
		'bgcolor': ((0, 0, 0, 0), tuple),
		'linecolor': ((225, 225, 225, 255), tuple),
		'textcolor': ((225, 175, 225, 255), tuple),
		'font': (font_path, str),
		'fontSize': (24, int),
		'format': ('base64', str),
		'imagepath': ('', str),
		'text': ('', str)
	}


	def __init__(self, **kwargs):
		data = {k: v[0] for k,v in self.defaults.items()}
		data.update(kwargs)

		if kwargs.get('imagepath'):
			kwargs['format'] = 'image'

		if self.word_cache_path_local.exists:
			with self.word_cache_path_local.open() as fd:
				self.word_cache = [v for v in fd.read().splitlines() if "'" not in v and len(v) <= 10]

		elif not self.word_cache_path.exists:
			resp = urlopen('https://svnweb.freebsd.org/csrg/share/dict/words?view=co&content-type=text/plain')
			data = resp.read().decode()

			with self.word_cache_path.open('w') as fd:
				fd.write(data)

			self.word_cache = data.splitlines()

		self.update(data)

		if not kwargs.get('text'):
			self.word_gen(1)


	def __str__(self):
		return f'Captcha(text={self.text})'


	def __getitem__(self, key):
		if key not in self.defaults:
			raise KeyError(f'Not a valid option: {key}')

		return getattr(self, key)


	def __setitem__(self, key, value):
		if key not in self.defaults:
			raise KeyError(f'Not a valid option: {key}')

		valid_type = self.defaults[key][1]

		if not isinstance(value, valid_type):
			raise TypeError(f'Value for {key} should be a {valid_type.__name__}, not a {value.__class__.__name__}')

		setattr(self, key, value)


	def word_gen(self, num=2):
		result = []

		for _ in range(num):
			result.append(random.choice(self.word_cache))

		self.text = ' '.join(result)


	def get(self, key, default=None):
		try:
			return self[key]

		except KeyError:
			return default


	def items(self):
		return ((key, self[key]) for key in self.defaults.keys())


	def values(self):
		return (v for k, v in self.items())


	def keys(self):
		return (k for k, v in self.items())


	def reset(self):
		for k, data in self.defaults.items():
			self[key] = data[0]


	def update(self, data):
		for k, v in data.items():
			self[k] = v


	def as_dict(self):
		return DotDict({k: v for k,v in self.items()})


	def to_json(self, indent=4):
		return self.as_dict().to_json(indent)
