import sys

from izzylib import logging
from izzylib.http_server import Application, View
from os import environ as env

from . import middleware, views, __software__, __version__
from .config import config, path
from .database import OperationalError, db, version


try:
	dbversion = db.get('config', 'version')

	if not dbversion:
		logging.error('Database has not been setup. Please run `make setup` first.')
		sys.exit()

	if dbversion < version:
		logging.error('Database needs to be migrated. Please run `make migrate`.')
		sys.exit()

except OperationalError as e:
	logging.error('Database has not been setup. Please run `make setup` first.')
	sys.exit()


app = Application(
	name = __software__,
	version = __version__,
	git_repo = 'https://git.barkshark.xyz/izaliamae/pyweb-sync',
	tpl_search = [path.frontend],
	access_log = True,
	**config
)

app.add_middleware(middleware.LoginDetect)


## Frontend
for cls in dir(views):
	view = getattr(views, cls)

	try:
		view.paths
		app.add_class_route(view)

	except AttributeError:
		pass

app.static('/static', path.frontend.join('static'))
app.static('/style', path.frontend.join('style'))


## Api
# in the works #
